/**
 * Problem: maxcut3d
 * Correct solution (must be OK)
 * Author: stgatilov
 * Sweeping plane algorithm with time = Z.
 * Quadratic functions of area from triangle section are stored in segment tree.
 * These area values are zero when triangle is inactive, and are set properly while it is active.
 * Time: O(N log N)
 */

//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

#ifndef MAXCUT3D_COMMON_H_236554
#define MAXCUT3D_COMMON_H_236554

#include <stdarg.h>
#include <vector>
#include <map>
#include <string>
#include <algorithm>
using namespace std;

typedef long long int64;

template<class T> T myabs(T x) { return x < 0 ? -x : x; }

//=====================================================================================

template<class Elem> struct Point {
    union {
        Elem a[3];
        struct { Elem x, y, z; };
    };

    explicit Point(Elem _c = 0) {
        x = _c;
        y = _c;
        z = _c;
    }
    Point(Elem _x, Elem _y, Elem _z) {
        x = _x;
        y = _y;
        z = _z;
    }
    Elem operator[](int i) const { return a[i]; }
    Elem& operator[](int i) { return a[i]; }
    bool IsZero() const { return x == 0 && y == 0 && z == 0; }

    Point(const Point &) = default;
    template<class T> explicit Point(const Point<T> &p) {
        x = Elem(p.x);
        y = Elem(p.y);
        z = Elem(p.z);
    }
    Point(const typename conditional<is_same<Elem, int>::value, void******, Point<int>>::type &p) {
        x = Elem(p.x);
        y = Elem(p.y);
        z = Elem(p.z);
    }

    string str() const {
        return "(" + to_string(x) + "," + to_string(y) + "," + to_string(z) + ")";
    }
};
typedef Point<int> IPoint;
typedef Point<double> DPoint;
typedef Point<int64> I64Point;

template<class Elem> Point<Elem> operator+ (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(a.x + b.x, a.y + b.y, a.z + b.z);
}
template<class Elem> Point<Elem> operator- (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(a.x - b.x, a.y - b.y, a.z - b.z);
}
template<class Elem> Point<Elem> operator- (const Point<Elem> &b) {
    return Point<Elem>(-b.x, -b.y, -b.z);
}
template<class Elem> Point<Elem> operator* (const Point<Elem> &a, Elem b) {
    return Point<Elem>(a.x * b, a.y * b, a.z * b);
}
template<class Elem> Point<Elem> operator* (Elem b, const Point<Elem> &a) {
    return Point<Elem>(a.x * b, a.y * b, a.z * b);
}
template<class Elem> Point<Elem> operator/ (const Point<Elem> &a, Elem b) {
    return Point<Elem>(a.x / b, a.y / b, a.z / b);
}
template<class Elem> Point<Elem> operator* (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(a.x * b.x, a.y * b.y, a.z * b.z);
}
template<class Elem> Point<Elem> operator/ (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(a.x / b.x, a.y / b.y, a.z / b.z);
}

template<class Elem> void operator+= (Point<Elem> &a, const Point<Elem> &b) { a = a + b; }
template<class Elem> void operator-= (Point<Elem> &a, const Point<Elem> &b) { a = a - b; }
template<class Elem> void operator*= (Point<Elem> &a, Elem b) { a = a * b; }
template<class Elem> void operator/= (Point<Elem> &a, Elem b) { a = a / b; }
template<class Elem> void operator*= (Point<Elem> &a, const Point<Elem> &b) { a = a * b; }
template<class Elem> void operator/= (Point<Elem> &a, const Point<Elem> &b) { a = a / b; }

template<class Elem> Elem dot (const Point<Elem> &a, const Point<Elem> &b) {
    return a.x * b.x + a.y * b.y + a.z * b.z;
}
template<class Elem> Point<Elem> cross (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(
        a.y * b.z - a.z * b.y,
        a.z * b.x - a.x * b.z,
        a.x * b.y - a.y * b.x
    );
}
inline Point<int64> cross (const Point<int64> &a, const Point<int64> &b) {
    return Point<int64>(
        int64(a.y) * b.z - int64(a.z) * b.y,
        int64(a.z) * b.x - int64(a.x) * b.z,
        int64(a.x) * b.y - int64(a.y) * b.x
    );
}
inline int64 dot (const IPoint &a, const IPoint &b) { return dot(I64Point(a), I64Point(b)); }
inline int64 dot (const I64Point &a, const IPoint &b) { return dot(I64Point(a), I64Point(b)); }
inline int64 dot (const IPoint &a, const I64Point &b) { return dot(I64Point(a), I64Point(b)); }
inline I64Point cross (const IPoint &a, const IPoint &b) { return cross(I64Point(a), I64Point(b)); }
inline I64Point cross (const I64Point &a, const IPoint &b) { return cross(I64Point(a), I64Point(b)); }
inline I64Point cross (const IPoint &a, const I64Point &b) { return cross(I64Point(a), I64Point(b)); }

template<class Elem> Point<Elem> pinf (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(min(a.x, b.x), min(a.y, b.y), min(a.z, b.z));
}
template<class Elem> Point<Elem> psup (const Point<Elem> &a, const Point<Elem> &b) {
    return Point<Elem>(max(a.x, b.x), max(a.y, b.y), max(a.z, b.z));
}

template<class Elem> Elem uninorm(const Point<Elem> &a) {
    return max(max(myabs(a.x), myabs(a.y)), myabs(a.z));
}

template<class Elem> bool operator== (const Point<Elem> &a, const Point<Elem> &b) {
    return a.x == b.x && a.y == b.y && a.z == b.z;
}

template<class Elem> struct CmpLex {
    bool operator() (const Point<Elem> &a, const Point<Elem> &b) const {
        if (a.x != b.x) return a.x < b.x;
        if (a.y != b.y) return a.y < b.y;
        if (a.z != b.z) return a.z < b.z;
        return false;
    }
};

//=====================================================================================

template<class Elem> struct Box;

template<class Elem> struct Mesh {
    vector<Point<Elem>> verts;
    vector<IPoint> tris;

    int size() const { return tris.size(); }
    Box<Elem> GetBox(int idx) const;
};
typedef Mesh<int> IMesh;
typedef Mesh<double> DMesh;

//=====================================================================================

inline void PrintInputMesh(const IMesh &mesh) {
    printf("%d %d\n", int(mesh.verts.size()), int(mesh.tris.size()));
    for (IPoint p : mesh.verts)
        printf("%d %d %d\n", p.x, p.y, p.z);
    for (IPoint t : mesh.tris)
        printf("%d %d %d\n", t.x, t.y, t.z);
}

inline IMesh ReadInputMesh(const char *filename) {
    FILE *f = fopen(filename, "rt");
    int v, t;
    fscanf(f, "%d%d", &v, &t);
    IMesh mesh;
    mesh.verts.resize(v);
    mesh.tris.resize(t);
    for (IPoint &p : mesh.verts)
        fscanf(f, "%d%d%d\n", &p.x, &p.y, &p.z);
    for (IPoint &t : mesh.tris)
        fscanf(f, "%d%d%d\n", &t.x, &t.y, &t.z);
    fclose(f);
    return mesh;
}

#endif

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

IMesh mesh;

double bestArea = 0.0, bestZ = 0.0;
int bestLeft, bestRight;
double bestRatio;

typedef pair<DPoint, DPoint> MovingPoint;
typedef DPoint QuadricFunction;

MovingPoint GetMovingVertex(int a, int b, int z0) {
    IPoint beg = mesh.verts[a];
    IPoint end = mesh.verts[b];
    assert(beg.z != end.z);
    double invDZ = 1.0 / (end.z - beg.z);
    return MovingPoint(
        DPoint(beg) + DPoint(end - beg) * ((z0 - beg.z) * invDZ),
        DPoint(end - beg) * invDZ
    );
}

QuadricFunction AreaDoubled(const MovingPoint &a, const MovingPoint &b) {
    QuadricFunction res;
    res[0]  =  a.first.x *  b.first.y -  a.first.y *  b.first.x;
    res[1]  =  a.first.x * b.second.y -  a.first.y * b.second.x;
    res[1] += a.second.x *  b.first.y - a.second.y *  b.first.x;
    res[2]  = a.second.x * b.second.y - a.second.y * b.second.x;
    return res;
}

double EvalFunc(const QuadricFunction &f, double z) {
    return ((f[2] * z) + f[1]) * z + f[0];
}


void RelaxFuncEval(const QuadricFunction &f, int lz, int rz, double ez) {
    if (ez < lz || ez > rz)
        return;
    double area = EvalFunc(f, ez);
    if (bestArea < area) {
        bestArea = area;
        bestZ = ez;
        bestLeft = lz;
        bestRight = rz;
        bestRatio = (ez-lz) / (rz-lz);
    }
}

struct Event {
    int z;
    int idx;
};
bool operator< (const Event &a, const Event &b) {
    if (a.z != b.z) return a.z < b.z;
    return a.idx < b.idx;
}

int trN;
vector<QuadricFunction> areaTree;

void SetTree(int pos, const QuadricFunction &value) {
    int idx = trN-1 + pos;
    areaTree[idx] = value;
    while (idx > 0) {
        idx = (idx-1) >> 1;
        areaTree[idx] = areaTree[2*idx+1] + areaTree[2*idx+2];
    }
}

vector<vector<int>> trisWithVertex;
vector<char> active;

void AddTri(int idx, int sign, int currZ) {
    if (sign < 0 && active[idx]) {
        active[idx] = false;
        SetTree(idx, QuadricFunction(0.0));
    }
    else if (sign > 0 && !active[idx]) {
        active[idx] = true;
        IPoint tri = mesh.tris[idx];

        char vertexTop[3] = {false};
        for (int i = 0; i < 3; i++) {
            int vertexZ = mesh.verts[tri[i]].z;
            vertexTop[i] = (vertexZ > currZ);
        }

        pair<int, int> beg{-1, -1}, end{-1, -1};
        int num = 0;
        for (int j = 0; j < 3; j++) {
            if (vertexTop[j] == vertexTop[(j+1)%3])
                continue;
            int a = tri[j], b = tri[(j+1)%3];
            (vertexTop[j] ? beg : end) = make_pair(a, b);
            num++;
        }
        assert(num % 2 == 0);

        if (num == 2) {
            assert(beg.first >= 0 && end.first >= 0);
            MovingPoint bxy = GetMovingVertex(beg.first, beg.second, 0);
            MovingPoint exy = GetMovingVertex(end.first, end.second, 0);
            QuadricFunction df = AreaDoubled(bxy, exy);
            SetTree(idx, df);
        }
    }
}

int main(int argc, char **argv) {
    //freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);
    mesh = ReadInputMesh("input.txt");

    int v = mesh.verts.size();
    int t = mesh.tris.size();
    vector<Event> evs;
    for (int i = 0; i < v; i++)
        evs.push_back(Event{mesh.verts[i].z, i});
    sort(evs.begin(), evs.end());

    trisWithVertex.assign(v, {});
    for (int i = 0; i < t; i++) {
        IPoint tri = mesh.tris[i];
        for (int j = 0; j < 3; j++)
            trisWithVertex[tri[j]].push_back(i);
    }

    trN = 1;
    while (trN < t) trN *= 2;
    areaTree.assign(trN*2-1, QuadricFunction(0.0));
    active.assign(t, false);

    int done = 0;
    while (done < v) {
        int secZ = evs[done].z;
        int from = done;
        while (done < v && evs[done].z == secZ)
            done++;

        for (int sign = -1; sign <= 1; sign += 2)
            for (int i = from; i < done; i++)
                for (int idx : trisWithVertex[evs[i].idx])
                    AddTri(idx, sign, secZ);

        if (done < v) {
            QuadricFunction func = areaTree[0];
            int lz = evs[from].z, rz = evs[done].z;
            assert(lz < rz);
            RelaxFuncEval(func, lz, rz, lz);
            RelaxFuncEval(func, lz, rz, rz);
            if (fabs(func[2]) > 1e-100) {
                double Zextr = -func[1] / (2.0 * func[2]);
                RelaxFuncEval(func, lz, rz, Zextr);
            }
        }
    }

    bestArea *= 0.5;
    Ef("Best: Z = %0.17g, Area = %0.17g; [%d..%d] with ratio %0.15lf\n", bestZ, bestArea, bestLeft, bestRight, bestRatio);
    printf("%0.17g %0.17g\n", bestZ, bestArea);

    return 0;
}
