/**
 * Problem: woodcut
 * Correct solution (must be OK)
 * Author: stgatilov
 * Draw ring of possible car location for each tree.
 * Check all circle intersections and one random point on every circle.
 * Time: O(N^3)
 */

//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

const int MAXN = 512;
const double PI = 3.1415926535897932384626433832795;

struct Point {
    double x, y;

    Point() : x(0), y(0) {}
    Point(double _x, double _y) : x(_x), y(_y) {}
};

Point operator+ (Point a, Point b) { return Point(a.x + b.x, a.y + b.y); }
Point operator- (Point a, Point b) { return Point(a.x - b.x, a.y - b.y); }
Point operator* (Point a, double b) { return Point(a.x * b, a.y * b); }
double dot(Point a, Point b) { return a.x * b.x + a.y * b.y; }
double length(Point a) { return sqrt(dot(a, a)); }

int IntersectCircles(Point ac, double ar, Point bc, double br, Point *inters) {
    //implicit line:
    //  dot(coeff, P - ac) = dist
    Point coeff = bc - ac;
    double cl2 = dot(coeff, coeff);
    double dist = ((ar*ar - br*br) + cl2) * 0.5;
    //parametric line:
    //  P(t) = ac + [start + dir * t]
    Point start = coeff * (dist / cl2);
    Point dir(-coeff.y, coeff.x);
    //quadratic equation:
    //  length(P(t) - ac)^2 = ar^2
    //  qa t^2 + 2 qb t + qc = 0
    double qa = cl2;
    double qb = 0.0;//dot(coeff, dir);
    double qc = dot(start, start) - ar*ar;
    //solve it
    double qd = qb*qb - qa*qc;
    if (qc > 1e-4)
        return 0;
    qd = max(qd, 0.0);
    qd = sqrt(qd);
    //output points
    int k = (qd == 0.0 ? 1 : 2);
    for (int i = 0; i < k; i++) {
        double param = (-qb + (i==0 ? 1 : -1) * qd) / qa;
        inters[i] = (start + dir * param) + ac;
    }
    return k;
}

int n;
double width;
Point ctr[MAXN];
double hgt[MAXN];

Point bestPoint;
int bestMultiplicity;

int lastIds[MAXN];
int checkCount;
void CheckPoint(Point car) {
    checkCount++;
    static const double RADIUS_EPS = 0.5e-6;

    int mult = 0;
    for (int i = 0; i < n; i++) {
        Point tc = ctr[i];
        double th = hgt[i];
        double dist = length(car - tc);

        if (dist > th + width + RADIUS_EPS)
            continue;
        if (dist < th - width - RADIUS_EPS)
            continue;
        lastIds[mult++] = i;
    }

    if (bestMultiplicity < mult) {
        bestMultiplicity = mult;
        bestPoint = car;
    }
}

int main(int argc, char **argv) {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);

    scanf("%d%lf", &n, &width);
    for (int i = 0; i < n; i++)
        scanf("%lf%lf%lf", &ctr[i].x, &ctr[i].y, &hgt[i]);

    for (int i = 0; i < n; i++) {
        CheckPoint(ctr[i] + Point(0.8, 0.6) * (hgt[i] - width));
        CheckPoint(ctr[i] + Point(0.6, 0.8) * (hgt[i] + width));
    }
    for (int i = 0; i < n; i++)
        for (int j = 0; j < i; j++)
            for (int si = -1; si <= 1; si += 2)
                for (int sj = -1; sj <= 1; sj += 2) {
                    Point inters[2];
                    int k = IntersectCircles(
                        ctr[i], hgt[i] + si * width,
                        ctr[j], hgt[j] + sj * width,
                        inters
                    );
                    for (int u = 0; u < k; u++)
                        CheckPoint(inters[u]);
                }

    CheckPoint(bestPoint);
    srand(12345);
    random_shuffle(lastIds, lastIds + bestMultiplicity); //just for lulz

    printf("%d\n", bestMultiplicity);
    printf("%0.16lf %0.16lf\n", bestPoint.x, bestPoint.y);
    for (int i = 0; i < bestMultiplicity; i++) {
        Point dir = bestPoint - ctr[lastIds[i]];
        double angle = atan2(dir.y, dir.x);
        if (angle < 0.0) angle += 2*PI;
        printf("%d %0.16lf\n", lastIds[i] + 1, angle / PI * 180.0);
    }
    Ef("Best %d/%d, checked %d points\n", bestMultiplicity, n, checkCount);

    return 0;
}
