/**
 * Problem: intcurve
 * Correct solution (must be OK)
 * Author: stgatilov
 * Dynamic programming in O(N^2).
 */

//#pragma comment(linker, "/STACK:20000000")
#include <vector>
#include <list>
#include <map>
#include <set>
#include <deque>
#include <stack>
#include <bitset>
#include <algorithm>
#include <functional>
#include <numeric>
#include <utility>
#include <sstream>
#include <iostream>
#include <iomanip>
#include <cstdio>
#include <cmath>
#include <cstring>
#include <cassert>
#include <cstdlib>
#include <ctime>
#include <cstdint>
#include <unordered_set>
#include <cinttypes>
#include <array>
#include <memory>
#include <climits>

using namespace std;
typedef long long int64;
#ifdef HOME
    #define E(c) cerr<<#c
    #define Eo(x) cerr<<#x<<" = "<<(x)<<endl
    #define Ef(...) fprintf(stderr, __VA_ARGS__)
#else
    #define E(c) ((void)0)
    #define Eo(x) ((void)0)
    #define Ef(...) ((void)0)
#endif

const int MAXN = 20<<10;

struct Oper {
    double delta, size;
    double c, d;
    int name;
};
bool cmpSize (const Oper &a, const Oper &b) {
    if (a.size != b.size)
        return a.size < b.size;
    return a.name < b.name;
}
bool cmpName (const Oper &a, const Oper &b) {
    return a.name < b.name;
}

int n;
double s, a, b;
Oper opers[MAXN];

double dp[MAXN];
int parState[MAXN], parDelta[MAXN], parFirst[MAXN];
double sol[MAXN];
void Solve(int mode) {
    if (mode == 2)
        sort(opers, opers+n, cmpSize);
    else
        assert(is_sorted(opers, opers+n, cmpName));

    for (int i = 0; i <= n; i++)
        dp[i] = 1e+100;
    dp[0] = 0.0;

    for (int i = 0; i < n; i++) {
        int maxName = -1;
        double maxSize = 0.0;
        int earliest = INT_MAX;
        double sumC = 0.0, sumD = 0.0;

        for (int j = i+1; j <= n; j++) {
            if (mode == 0 && j-i > 1)
                break;
            const Oper &last = opers[j-1];
            if (maxSize < last.size) {
                maxSize = last.size;
                maxName = last.name;
            }
            if (earliest > last.name)
                earliest = last.name;
            sumC += last.c;
            sumD += last.d;

            double tcost = (a * maxSize + b) + (sumC * maxSize + sumD);
            double nres = dp[i] + tcost;
            if (dp[j] > nres) {
                dp[j] = nres;
                parState[j] = i;
                parDelta[j] = maxName;
                parFirst[j] = earliest;
            }
        }
    }
    double ans = dp[n];

    if (mode == 2)
        sort(opers, opers+n, cmpName);

    for (int i = 0; i < n; i++)
        sol[i] = -1.0;
    int curr = n;
    int k = 0;
    while (curr > 0) {
        int prev = parState[curr];
        double eps = opers[parDelta[curr]].delta;
        int idx = parFirst[curr];
        sol[idx] = eps;
        curr = prev;
        k++;
    }
    Ef("mode%d: %0.8g with %d/%d builds\n", mode, ans, k, n);

    printf("%0.15g\n", ans);
    for (int i = 0; i < n; i++)
        printf("%0.15g\n", sol[i]);
}

int main(int argc, char **argv) {
    freopen("input.txt", "r", stdin);
    freopen("output.txt", "w", stdout);

    scanf("%d", &n);
    scanf("%lf%lf%lf", &s, &a, &b);
    for (int i = 0; i < n; i++) {
        Oper op;
        scanf("%lf%lf%lf", &op.delta, &op.c, &op.d);
        op.name = i;
        op.size = s / pow(op.delta, 0.25);
        opers[i] = op;
    }

    for (int m = 0; m < 3; m++) {
        if (m) printf("===\n");
        Solve(m);
    }

    return 0;
}
